@@ -423,7 +423,6 @@ plot_versions <- function(start_p, end_p, path_p) {
 
 write_versions <- function(start_p = NULL, end_p = NULL, path_p) {
   prepare_versions(start_p, end_p) %>%
-    spread(key = "version", value = "relays", fill = 0) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -459,6 +458,7 @@ plot_platforms <- function(start_p, end_p, path_p) {
 
 write_platforms <- function(start_p = NULL, end_p = NULL, path_p) {
   prepare_platforms(start_p, end_p) %>%
+    mutate(platform = tolower(platform)) %>%
     spread(platform, relays) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
@@ -574,7 +574,7 @@ write_dirbytes <- function(start_p = NULL, end_p = NULL, path_p) {
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
-prepare_relayflags <- function(start_p, end_p, flags_p) {
+prepare_relayflags <- function(start_p, end_p, flag_p) {
   read.csv(paste(stats_dir, "servers.csv", sep = ""),
     colClasses = c("date" = "Date")) %>%
     filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
@@ -584,12 +584,12 @@ prepare_relayflags <- function(start_p, end_p, flags_p) {
     filter(platform == "") %>%
     filter(ec2bridge == "") %>%
     mutate(flag = ifelse(flag == "", "Running", as.character(flag))) %>%
-    filter(if (!is.null(flags_p)) flag %in% flags_p else TRUE) %>%
+    filter(if (!is.null(flag_p)) flag %in% flag_p else TRUE) %>%
     select(date, flag, relays)
 }
 
-plot_relayflags <- function(start_p, end_p, flags_p, path_p) {
-  prepare_relayflags(start_p, end_p, flags_p) %>%
+plot_relayflags <- function(start_p, end_p, flag_p, path_p) {
+  prepare_relayflags(start_p, end_p, flag_p) %>%
     complete(date = full_seq(date, period = 1), flag = unique(flag)) %>%
     ggplot(aes(x = date, y = relays, colour = as.factor(flag))) +
     geom_line() +
@@ -598,17 +598,15 @@ plot_relayflags <- function(start_p, end_p, flags_p, path_p) {
     scale_y_continuous(name = "", labels = formatter, limits = c(0, NA)) +
     scale_colour_manual(name = "Relay flags", values = c("#E69F00",
         "#56B4E9", "#009E73", "#EE6A50", "#000000", "#0072B2"),
-        breaks = flags_p, labels = flags_p) +
+        breaks = flag_p, labels = flag_p) +
     ggtitle("Number of relays with relay flags assigned") +
     labs(caption = copyright_notice)
   ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
 }
 
-write_relayflags <- function(start_p = NULL, end_p = NULL, flags_p = NULL,
+write_relayflags <- function(start_p = NULL, end_p = NULL, flag_p = NULL,
     path_p) {
-  prepare_relayflags(start_p, end_p, flags_p) %>%
-    mutate(flag = tolower(flag)) %>%
-    spread(flag, relays) %>%
+  prepare_relayflags(start_p, end_p, flag_p) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -721,6 +719,81 @@ write_torperf_failures <- function(start_p = NULL, end_p = NULL,
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
+prepare_onionperf_buildtimes <- function(start_p, end_p, source_p) {
+    read.csv(paste(stats_dir, "buildtimes.csv", sep = ""),
+    colClasses = c("date" = "Date")) %>%
+    filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
+    filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
+    filter(if (!is.null(source_p))
+        source == ifelse(source_p == "all", "", source_p) else TRUE)
+}
+
+write_onionperf_buildtimes <- function(start_p = NULL, end_p = NULL,
+    source_p = NULL, path_p) {
+  prepare_onionperf_buildtimes(start_p, end_p, source_p) %>%
+    write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
+}
+
+plot_onionperf_buildtimes <- function(start_p, end_p, source_p, path_p) {
+  prepare_onionperf_buildtimes(start_p, end_p, source_p) %>%
+    mutate(date = as.Date(date),
+      position = factor(position, levels = seq(1, 3, 1),
+        labels = c("1st hop", "2nd hop", "3rd hop"))) %>%
+    ggplot(aes(x = date, y = md, colour = position, fill = position)) +
+    geom_line(size = 0.75) +
+    geom_ribbon(aes(x = as.Date(date), ymin = q1, ymax = q3, alpha = 0.5),
+      show.legend = FALSE) +
+    scale_x_date(name = "", breaks = custom_breaks,
+      labels = custom_labels, minor_breaks = custom_minor_breaks) +
+    scale_y_continuous(name = "", labels = unit_format(unit = "ms"),
+      limits = c(0, NA)) +
+    scale_colour_hue(name = "Medians and interquartile ranges") +
+    scale_fill_hue(name = "Medians and interquartile ranges") +
+    ggtitle(ifelse(source_p == "all", "Circuit build times on all sources",
+        paste("Circuit build times on", source_p))) +
+    labs(caption = copyright_notice) +
+    theme(legend.position = "top")
+  ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
+}
+
+prepare_onionperf_latencies <- function(start_p, end_p, source_p) {
+    read.csv(paste(stats_dir, "latencies.csv", sep = ""),
+    colClasses = c("date" = "Date")) %>%
+    filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
+    filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
+    filter(if (!is.null(source_p))
+        source == ifelse(source_p == "all", "", source_p) else TRUE)
+}
+
+write_onionperf_latencies <- function(start_p = NULL, end_p = NULL,
+    source_p = NULL, path_p) {
+  prepare_onionperf_latencies(start_p, end_p, source_p) %>%
+    write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
+}
+
+plot_onionperf_latencies <- function(start_p, end_p, source_p, path_p) {
+  prepare_onionperf_latencies(start_p, end_p, source_p) %>%
+    mutate(date = as.Date(date),
+      server = factor(server, levels = c("public", "onion"),
+        labels = c("public server", "onion server"))) %>%
+    ggplot(aes(x = date, y = md, colour = server, fill = server)) +
+    geom_line(size = 0.75) +
+    geom_ribbon(aes(x = as.Date(date), ymin = q1, ymax = q3, alpha = 0.5),
+      show.legend = FALSE) +
+    scale_x_date(name = "", breaks = custom_breaks,
+      labels = custom_labels, minor_breaks = custom_minor_breaks) +
+    scale_y_continuous(name = "", labels = unit_format(unit = "ms"),
+      limits = c(0, NA)) +
+    scale_colour_hue(name = "Medians and interquartile ranges") +
+    scale_fill_hue(name = "Medians and interquartile ranges") +
+    ggtitle(ifelse(source_p == "all",
+        "Circuit round-trip latencies on all sources",
+        paste("Circuit round-trip latencies on", source_p))) +
+    labs(caption = copyright_notice) +
+    theme(legend.position = "top")
+  ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
+}
+
 prepare_connbidirect <- function(start_p, end_p) {
   read.csv(paste(stats_dir, "connbidirect2.csv", sep = ""),
     colClasses = c("date" = "Date", "direction" = "factor")) %>%
@@ -757,9 +830,6 @@ plot_connbidirect <- function(start_p, end_p, path_p) {
 write_connbidirect <- function(start_p = NULL, end_p = NULL, path_p) {
   prepare_connbidirect(start_p, end_p) %>%
     rename(q1 = X0.25, md = X0.5, q3 = X0.75) %>%
-    gather(variable, value, -(date:direction)) %>%
-    unite(temp, direction, variable) %>%
-    spread(temp, value) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -825,7 +895,8 @@ plot_userstats <- function(start_p, end_p, node_p, variable_p, value_p,
     events_p, path_p) {
   load(paste(rdata_dir, "clients-", node_p, ".RData", sep = ""))
   c <- data
-  u <- c[c$date >= start_p & c$date <= end_p, ]
+  u <- c[c$date >= start_p & c$date <= end_p, c("date", "country", "transport",
+      "version", "lower", "upper", "clients")]
   u <- rbind(u, data.frame(date = start_p,
       country = ifelse(variable_p == "country" & value_p != "all", value_p, ""),
       transport = ifelse(variable_p == "transport", value_p, ""),
@@ -978,8 +1049,7 @@ write_userstats_relay_country <- function(start_p = NULL, end_p = NULL,
       country == ifelse(country_p == "all", "", country_p) else TRUE) %>%
     filter(transport == "") %>%
     filter(version == "") %>%
-    mutate(downturns = clients < lower, upturns = clients > upper) %>%
-    select(date, country, clients, downturns, upturns, lower, upper) %>%
+    select(date, country, clients, lower, upper, frac) %>%
     rename(users = clients) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
@@ -994,7 +1064,7 @@ write_userstats_bridge_country <- function(start_p = NULL, end_p = NULL,
       country == ifelse(country_p == "all", "", country_p) else TRUE) %>%
     filter(transport == "") %>%
     filter(version == "") %>%
-    select(date, country, clients) %>%
+    select(date, country, clients, frac) %>%
     rename(users = clients) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
@@ -1008,24 +1078,21 @@ write_userstats_bridge_transport <- function(start_p = NULL, end_p = NULL,
     filter(country == "") %>%
     filter(version == "") %>%
     filter(transport != "") %>%
-    select(date, transport, clients)
+    select(date, transport, clients, frac)
   if (is.null(transport_p) || "!<OR>" %in% transport_p) {
     n <- u %>%
       filter(transport != "<OR>") %>%
-      group_by(date) %>%
+      group_by(date, frac) %>%
       summarize(clients = sum(clients))
     u <- rbind(u, data.frame(date = n$date, transport = "!<OR>",
-                             clients = n$clients))
+                             clients = n$clients, frac = n$frac))
   }
   u %>%
     filter(if (!is.null(transport_p)) transport %in% transport_p else TRUE) %>%
-    mutate(transport = ifelse(transport == "<OR>", "default_or_protocol",
-      ifelse(transport == "!<OR>", "any_pt",
-      ifelse(transport == "<??>", "unknown_pluggable_transports",
-      transport)))) %>%
     group_by(date, transport) %>%
-    select(date, transport, clients) %>%
-    spread(transport, clients) %>%
+    select(date, transport, clients, frac) %>%
+    rename(users = clients) %>%
+    arrange(date, transport) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -1038,7 +1105,7 @@ write_userstats_bridge_version <- function(start_p = NULL, end_p = NULL,
     filter(country == "") %>%
     filter(transport == "") %>%
     filter(if (!is.null(version_p)) version == version_p else TRUE) %>%
-    select(date, version, clients) %>%
+    select(date, version, clients, frac) %>%
     rename(users = clients) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
@@ -1084,12 +1151,8 @@ write_userstats_bridge_combined <- function(start_p = NULL, end_p = NULL,
     write_userstats_bridge_country(start_p, end_p, country_p, path_p)
   } else {
     prepare_userstats_bridge_combined(start_p, end_p, country_p) %>%
-      select(date, country, transport, low, high) %>%
-      mutate(transport = ifelse(transport == "<OR>", "default_or_protocol",
-        ifelse(transport == "<??>", "unknown_transport", transport))) %>%
-      gather(variable, value, -(date:transport)) %>%
-      unite(temp, transport, variable) %>%
-      spread(temp, value) %>%
+      select(date, country, transport, low, high, frac) %>%
+      arrange(date, country, transport) %>%
       write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
   }
 }
@@ -1126,8 +1189,8 @@ plot_advbwdist_perc <- function(start_p, end_p, p_p, path_p) {
 write_advbwdist_perc <- function(start_p = NULL, end_p = NULL, p_p = NULL,
     path_p) {
   prepare_advbwdist_perc(start_p, end_p, p_p) %>%
-    unite(temp, variable, percentile) %>%
-    spread(temp, advbw) %>%
+    spread(variable, advbw) %>%
+    rename(p = percentile) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -1163,8 +1226,8 @@ plot_advbwdist_relay <- function(start_p, end_p, n_p, path_p) {
 write_advbwdist_relay <- function(start_p = NULL, end_p = NULL, n_p = NULL,
     path_p) {
   prepare_advbwdist_relay(start_p, end_p, n_p) %>%
-    unite(temp, variable, relay) %>%
-    spread(temp, advbw) %>%
+    spread(variable, advbw) %>%
+    rename(n = relay) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -1174,7 +1237,7 @@ prepare_hidserv_dir_onions_seen <- function(start_p, end_p) {
     filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     filter(type == "dir-onions-seen") %>%
-    transmute(date = date, onions = ifelse(frac >= 0.01, wiqm, NA))
+    transmute(date, onions = ifelse(frac >= 0.01, wiqm, NA), frac)
 }
 
 plot_hidserv_dir_onions_seen <- function(start_p, end_p, path_p) {
@@ -1202,7 +1265,7 @@ prepare_hidserv_rend_relayed_cells <- function(start_p, end_p) {
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     filter(type == "rend-relayed-cells") %>%
     transmute(date,
-      relayed = ifelse(frac >= 0.01, wiqm * 8 * 512 / (86400 * 1e9), NA))
+      relayed = ifelse(frac >= 0.01, wiqm * 8 * 512 / (86400 * 1e9), NA), frac)
 }
 
 plot_hidserv_rend_relayed_cells <- function(start_p, end_p, path_p) {
@@ -1224,40 +1287,6 @@ write_hidserv_rend_relayed_cells <- function(start_p = NULL, end_p = NULL,
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
-prepare_hidserv_frac_reporting <- function(start_p, end_p) {
-  read.csv(paste(stats_dir, "hidserv.csv", sep = ""),
-    colClasses = c("date" = "Date")) %>%
-    filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
-    filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
-    select(date, frac, type)
-}
-
-plot_hidserv_frac_reporting <- function(start_p, end_p, path_p) {
-  prepare_hidserv_frac_reporting(start_p, end_p) %>%
-    ggplot(aes(x = date, y = frac, colour = type)) +
-    geom_line() +
-    geom_hline(yintercept = 0.01, linetype = 2) +
-    scale_x_date(name = "", breaks = custom_breaks,
-      labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = percent, limits = c(0, NA)) +
-    scale_colour_hue(name = "",
-                     breaks = c("rend-relayed-cells", "dir-onions-seen"),
-                     labels = c("Onion-service traffic",
-                                "Unique .onion addresses")) +
-    ggtitle(paste("Fraction of relays reporting onion-service",
-                       "statistics")) +
-    labs(caption = copyright_notice) +
-    theme(legend.position = "top")
-  ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
-}
-
-write_hidserv_frac_reporting <- function(start_p = NULL, end_p = NULL, path_p) {
-  prepare_hidserv_frac_reporting(start_p, end_p) %>%
-    mutate(type = ifelse(type == "dir-onions-seen", "onions", "relayed")) %>%
-    spread(type, frac) %>%
-    write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
-}
-
 prepare_webstats_tb <- function(start_p, end_p) {
   load(paste(rdata_dir, "webstats-tb.RData", sep = ""))
   data %>%
@@ -1365,22 +1394,14 @@ plot_webstats_tb_locale <- function(start_p, end_p, path_p) {
 # plot_webstats_tb_locale needs the preliminary data frame e for its
 # breaks and labels. Left as future work.
 write_webstats_tb_locale <- function(start_p = NULL, end_p = NULL, path_p) {
-  d <- read.csv(paste(stats_dir, "webstats.csv", sep = ""),
-    colClasses = c("log_date" = "Date", "locale" = "character"))
-  d <- d %>%
+  read.csv(paste(stats_dir, "webstats.csv", sep = ""),
+    colClasses = c("log_date" = "Date", "locale" = "character")) %>%
     filter(if (!is.null(start_p)) log_date >= as.Date(start_p) else TRUE) %>%
     filter(if (!is.null(end_p)) log_date <= as.Date(end_p) else TRUE) %>%
-    filter(request_type == "tbid")
-  e <- d
-  e <- aggregate(list(count = e$count), by = list(locale = e$locale), FUN = sum)
-  e <- e[order(e$count, decreasing = TRUE), ]
-  e <- e[1:5, ]
-  d <- aggregate(list(count = d$count), by = list(log_date = d$log_date,
-    locale = ifelse(d$locale %in% e$locale, d$locale, "other")), FUN = sum)
-  d %>%
-    mutate(locale = tolower(locale)) %>%
+    filter(request_type == "tbid") %>%
+    group_by(log_date, locale) %>%
+    summarize(initial_downloads = sum(count)) %>%
     rename(date = log_date) %>%
-    spread(locale, count) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
 
@@ -1414,7 +1435,8 @@ plot_webstats_tm <- function(start_p, end_p, path_p) {
 write_webstats_tm <- function(start_p = NULL, end_p = NULL, path_p) {
   prepare_webstats_tm(start_p, end_p) %>%
     rename(date = log_date) %>%
-    spread(request_type, count) %>%
+    mutate(request_type = factor(request_type, levels = c("tmid", "tmup"))) %>%
+    spread(request_type, count, drop = FALSE) %>%
     rename(initial_downloads = tmid, update_pings = tmup) %>%
     write.csv(path_p, quote = FALSE, row.names = FALSE, na = "")
 }
