@@ -10,6 +10,9 @@ import org.torproject.descriptor.ExtraInfoDescriptor;
 import org.torproject.descriptor.NetworkStatusEntry;
 import org.torproject.descriptor.RelayNetworkStatusConsensus;
 
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
 import java.io.File;
@@ -32,6 +35,8 @@ import java.util.TreeSet;
  * document files for later use. */
 public class Parser {
 
+  private static Logger log = LoggerFactory.getLogger(Parser.class);
+
   /** File containing tuples of last-modified times and file names of
    * descriptor files parsed in the previous execution. */
   private File parseHistoryFile;
@@ -106,14 +111,14 @@ public class Parser {
             String[] parts = line.split(" ", 2);
             excludedFiles.put(parts[1], Long.parseLong(parts[0]));
           } catch (NumberFormatException e) {
-            System.err.printf("Illegal line '%s' in parse history.  "
-                + "Skipping line.%n", line);
+            log.warn("Illegal line '{}' in parse history. Skipping line.", line,
+                e);
           }
         }
       } catch (IOException e) {
-        System.err.printf("Could not read history file '%s'.  Not "
+        log.warn("Could not read history file '{}'. Not "
             + "excluding descriptors in this execution.",
-            this.parseHistoryFile.getAbsolutePath());
+            this.parseHistoryFile.getAbsolutePath(), e);
       }
 
       /* Tell the descriptor reader to exclude the files contained in the
@@ -146,9 +151,8 @@ public class Parser {
             + "\n");
       }
     } catch (IOException e) {
-      System.err.printf("Could not write history file '%s'.  Not "
-          + "excluding descriptors in next execution.",
-          this.parseHistoryFile.getAbsolutePath());
+      log.warn("Could not write history file '{}'. Not excluding descriptors "
+          + "in next execution.", this.parseHistoryFile.getAbsolutePath(), e);
     }
   }
 
@@ -195,19 +199,10 @@ public class Parser {
     /* Extract the fingerprint from the parsed descriptor. */
     String fingerprint = extraInfoDescriptor.getFingerprint();
 
-    /* If the descriptor did not contain any of the expected hidserv-*
-     * lines, don't do anything.  This applies to the majority of
-     * descriptors, at least as long as only a minority of relays reports
-     * these statistics. */
-    if (extraInfoDescriptor.getHidservStatsEndMillis() < 0L
-        && extraInfoDescriptor.getHidservRendRelayedCells() == null
-        && extraInfoDescriptor.getHidservDirOnionsSeen() == null) {
-      return;
-
     /* If the descriptor contained all expected hidserv-* lines, create a
      * new stats object and put it in the local map, so that it will later
      * be written to a document file. */
-    } else if (extraInfoDescriptor.getHidservStatsEndMillis() >= 0L
+    if (extraInfoDescriptor.getHidservStatsEndMillis() >= 0L
         && extraInfoDescriptor.getHidservStatsIntervalLength() >= 0L
         && extraInfoDescriptor.getHidservRendRelayedCells() != null
         && extraInfoDescriptor.getHidservRendRelayedCellsParameters() != null
@@ -233,9 +228,11 @@ public class Parser {
      * out a warning.  This case does not warrant any further action,
      * because relays can in theory write anything in their extra-info
      * descriptors.  But maybe we'll want to know. */
-    } else {
-      System.err.println("Relay " + fingerprint + " published "
-          + "incomplete hidserv-stats.  Ignoring.");
+    } else if (extraInfoDescriptor.getHidservStatsEndMillis() >= 0L
+        || extraInfoDescriptor.getHidservRendRelayedCells() != null
+        || extraInfoDescriptor.getHidservDirOnionsSeen() != null) {
+      log.warn("Relay {} published incomplete hidserv-stats. Ignoring.",
+          fingerprint);
     }
   }
 
@@ -244,9 +241,7 @@ public class Parser {
   private long removeNoise(long reportedNumber, long binSize) {
     long roundedToNearestRightSideOfTheBin =
         Math.floorDiv((reportedNumber + binSize / 2), binSize) * binSize;
-    long subtractedHalfOfBinSize =
-        roundedToNearestRightSideOfTheBin - binSize / 2;
-    return subtractedHalfOfBinSize;
+    return roundedToNearestRightSideOfTheBin - binSize / 2;
   }
 
   /** Parses the given consensus. */
@@ -257,8 +252,8 @@ public class Parser {
     SortedMap<String, Integer> bandwidthWeights =
         consensus.getBandwidthWeights();
     if (bandwidthWeights == null) {
-      System.err.printf("Consensus with valid-after time %s doesn't "
-          + "contain any Wxx weights.  Skipping.%n",
+      log.warn("Consensus with valid-after time {} doesn't contain any Wxx "
+          + "weights. Skipping.",
           DateTimeHelper.format(consensus.getValidAfterMillis()));
       return;
     }
@@ -269,8 +264,8 @@ public class Parser {
         new TreeSet<>(Arrays.asList("Wmg,Wmm,Wme,Wmd".split(",")));
     expectedWeightKeys.removeAll(bandwidthWeights.keySet());
     if (!expectedWeightKeys.isEmpty()) {
-      System.err.printf("Consensus with valid-after time %s doesn't "
-          + "contain expected Wmx weights.  Skipping.%n",
+      log.warn("Consensus with valid-after time {} doesn't contain expected "
+          + "Wmx weights. Skipping.",
           DateTimeHelper.format(consensus.getValidAfterMillis()));
       return;
     }
