@@ -1,4 +1,4 @@
-/* Copyright 2017 The Tor Project
+/* Copyright 2017--2018 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.web;
@@ -13,11 +13,6 @@ public class MetricsLibServlet extends AnyServlet {
 
   private static final long serialVersionUID = -6009422570527820853L;
 
-  @Override
-  public void init() throws ServletException {
-    super.init();
-  }
-
   @Override
   public void doGet(HttpServletRequest request,
       HttpServletResponse response) throws IOException, ServletException {
